#ifndef __CLCD__
#define __CLCD__

#define _LCD_ROWS	4		/* Number of Rows (1,2 or 4) */
#define _LCD_COLS	20		/* Number of Columns (8..40) */
// cursor position to DDRAM mapping
#define LCD_LINE0_DDRAMADDR		0x00
#define LCD_LINE1_DDRAMADDR		0x40
#define LCD_LINE2_DDRAMADDR		0x14
#define LCD_LINE3_DDRAMADDR		0x54
#define LCD_DDRAM           7	//DB7: set DD RAM address

#define RSDDRNum		DDRA
#define RSPORTNum      PORTA
#define RSPINNum       PINA
#define RSBITNum         4
#define RSPORT(x)   ((x) ? (RSPORTNum |= (1 << RSBITNum) ) : (RSPORTNum &= (~(1 << RSBITNum)) ));
#define RSDDR(x)    ((x) ? (RSDDRNum |= (1 << RSBITNum) ) : (RSDDRNum &= (~(1 << RSBITNum)) ));

#define RWDDRNum		 DDRA
#define RWPORTNum      PORTA
#define RWPINNum       PINA
#define RWBITNum         5
#define RWPORT(x)   ((x) ? (RWPORTNum |= (1 << RWBITNum) ) : (RWPORTNum &= (~(1 << RWBITNum)) ));
#define RWDDR(x)    ((x) ? (RWDDRNum |= (1 << RWBITNum) ) : (RWDDRNum &= (~(1 << RWBITNum)) ));

#define ENDDRNum		 DDRC
#define ENPORTNum      PORTC
#define ENPINNum       PINC
#define ENBITNum         3
#define ENPORT(x)   ((x) ? (ENPORTNum |= (1 << ENBITNum) ) : (ENPORTNum &= (~(1 << ENBITNum)) ));
#define ENDDR(x)    ((x) ? (ENDDRNum |= (1 << ENBITNum) ) : (ENDDRNum &= (~(1 << ENBITNum)) ));

#define D4DDRNum		 DDRG
#define D4PORTNum      PORTG
#define D4PINNum       PING
#define D4BITNum         2
#define D4PORT(x)   ((x) ? (D4PORTNum |= (1 << D4BITNum) ) : (D4PORTNum &= (~(1 << D4BITNum)) ));
#define D4DDR(x)    ((x) ? (D4DDRNum |= (1 << D4BITNum) ) : (D4DDRNum &= (~(1 << D4BITNum)) ));

#define D5DDRNum		 DDRC
#define D5PORTNum      PORTC
#define D5PINNum       PINC
#define D5BITNum         7
#define D5PORT(x)   ((x) ? (D5PORTNum |= (1 << D5BITNum) ) : (D5PORTNum &= (~(1 << D5BITNum)) ));
#define D5DDR(x)    ((x) ? (D5DDRNum |= (1 << D5BITNum) ) : (D5DDRNum &= (~(1 << D5BITNum)) ));

#define D6DDRNum		 DDRC
#define D6PORTNum      PORTC
#define D6PINNum       PINC
#define D6BITNum         6
#define D6PORT(x)   ((x) ? (D6PORTNum |= (1 << D6BITNum) ) : (D6PORTNum &= (~(1 << D6BITNum)) ));
#define D6DDR(x)    ((x) ? (D6DDRNum |= (1 << D6BITNum) ) : (D6DDRNum &= (~(1 << D6BITNum)) ));

#define D7DDRNum		 DDRC
#define D7PORTNum      PORTC
#define D7PINNum       PINC
#define D7BITNum         5
#define D7PORT(x)   ((x) ? (D7PORTNum |= (1 << D7BITNum) ) : (D7PORTNum &= (~(1 << D7BITNum)) ));
#define D7DDR(x)    ((x) ? (D7DDRNum |= (1 << D7BITNum) ) : (D7DDRNum &= (~(1 << D7BITNum)) ));

#define set		 1
#define clear	 0
void send_command(int CMD);
void lcd_init (void);
void lcd_gotoxy (unsigned char col , unsigned char row);
void lcd_putchar(char a);
void lcd_puts(char *str);

static
uint8_t Row, Column;	/* Current cursor position */

void send_command(int CMD)
{

	if((CMD & 0x100) == 0x100) {RSPORT(set);}
	else {RSPORT(clear);}
	if((CMD & 0x80) == 0x80) {D7PORT(set);}
	else {D7PORT(clear);}
	if((CMD & 0x40) == 0x40) {D6PORT(set);}
	else {D6PORT(clear);}
	if((CMD & 0x20) == 0x20) {D5PORT(set);}
	else {D5PORT(clear);}
	if((CMD & 0x10) == 0x10) {D4PORT(set);}
	else {D4PORT(clear);}
	_delay_ms(1);
	ENPORT(set);
	_delay_ms(1);
	ENPORT(clear);
	_delay_ms(1);
	
	//Turn off All GPIO Pins
	RSPORT(clear);
	ENPORT(clear);
	D7PORT(clear);
	D6PORT(clear);
	D5PORT(clear);
	D4PORT(clear);

	if((CMD & 0x100) == 0x100) {RSPORT(set);}
	else {RSPORT(clear);	}
	if((CMD & 0x08) == 0x08) {D7PORT(set);}
	else {D7PORT(clear);}
	//High order D6
	if((CMD & 0x04) == 0x04) {D6PORT(set);}
	else {D6PORT(clear);}
	//High order D5
	if((CMD & 0x02) == 0x02) {D5PORT(set);}
	else {D5PORT(clear);}
	//High order D4
	if((CMD & 0x01) == 0x01) {D4PORT(set);}
	else {D4PORT(clear);}
	//Low order D7

	//Send Command by toggle EN pin
	_delay_ms(1);
	ENPORT(set);
	_delay_ms(1);
	ENPORT(clear);
	_delay_ms(1);
	
	//Turn off All GPIO Pins
	RSPORT(clear);
	ENPORT(clear);
	D7PORT(clear);
	D6PORT(clear);
	D5PORT(clear);
	D4PORT(clear);
	
}

void lcd_init (void)
{
	//Turn off All GPIO Pins
	RSDDR(set);
	RWDDR(set);
	ENDDR(set);
	D7DDR(set);
	D6DDR(set);
	D5DDR(set);
	D4DDR(set);
	
	RSPORT(clear);
	RWPORT(clear);
	ENPORT(clear);
	D7PORT(clear);
	D6PORT(clear);
	D5PORT(clear);
	D4PORT(clear);
	
	_delay_ms(500);
	send_command(0x33);
	send_command(0x32);
	
	//Function set (set font and lines)
	send_command(0x28);
	
	//On off control
	send_command(0x0C);
	
	//Entry mode set
	send_command(0x06);
	
	//Clear LCD
	send_command(0x01);
	
	//Return Home
	send_command(0x02);
}


void lcd_gotoxy (unsigned char col , unsigned char row)
{
	register uint8_t DDRAMAddr;
	Row = row; Column = col;

	// remap lines into proper order
	switch(row)
	{
		case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+col; break;
		case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+col; break;
		case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+col; break;
		case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+col; break;
		default: DDRAMAddr = LCD_LINE0_DDRAMADDR+col;
	}
	// set data address
	send_command(1<<LCD_DDRAM | DDRAMAddr);

}

void lcd_putchar(char chr)
{
	send_command ( (0x100) | chr) ;
	Column++;
}


void lcd_puts(char *str)
{
	int i;
	for(i=0;str[i]!='\0';i++)
	lcd_putchar(str[i]);
}

void lcd_clear (void)
{

	send_command(0x01);
	//Return Home
	send_command(0x02);
}


#endif
